`timescale 1ns / 1ps

////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer:
//
// Create Date:   15:44:16 10/08/2006
// Design Name:   spi_adc_tb
// Project Name:  3Estart
// Target Device:  
// Tool versions:  
// Description: 
//
// Verilog Test Fixture created by ISE for module: nor_flash
//
// Dependencies:
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
////////////////////////////////////////////////////////////////////////////////

module spi_adc_tb;

  // Inputs
  reg main_clk;
  reg [2:0] wADR_I;
  reg wSTB_I;
  reg [31:0] wDAT_I;
  reg wWE_I;
  reg reset;
  reg spi_miso;

  // Outputs
  wire spi_mosi;
  wire spi_ampcs;
  wire spi_sck;
  wire spi_daccs;
  wire [31:0] wDAT_O;
  wire wACK_O;
  wire spi_adcs;

  // Bidirs
  wire [15:0] sf_D_IO;

  // State machine
  reg [4:0] state;
  reg [33:0] ShiftOut;

  // Instantiate the Unit Under Test (UUT)
  spi_adc uut (
    .spi_mosi(spi_mosi), 
    .spi_ampcs(spi_ampcs), 
    .spi_sck(spi_sck), 
    .spi_miso(spi_miso), 
    .spi_adcs(spi_adcs),
    .spi_daccs(spi_daccs),
    .main_clk(main_clk), 
    .reset(reset), 
    .wADR_I(wADR_I), 
    .wSTB_I(wSTB_I), 
    .wDAT_I(wDAT_I), 
    .wDAT_O(wDAT_O), 
    .wACK_O(wACK_O), 
    .wWE_I(wWE_I)
  );



  initial begin
    // Initialize Inputs
    main_clk = 0;
    wADR_I = 0;
    wSTB_I = 0;
    wDAT_I = 0;
    wWE_I = 0;
    state = 0;
    reset = 1;

    // Wait 100 ns for global reset to finish
    #100;
    reset = 0;
    #100;

    // Add stimulus here
    state = 11;        // Write cycle - AMP
    wDAT_I = 32'h000000a5;
    wADR_I = 3'd0;
    #2000;

    // Add stimulus here
    state = 11;        // Write cycle - DAC
    wDAT_I = 32'hAAAA5555;
    wADR_I = 3'd2;
    
    #2000;
    ShiftOut = {2'bzz, 14'b10101101001101, 2'bzz, 14'b11110000111100, 2'bzz};
    wADR_I = 3'd1;
    state = 1;        // Read cycle
  end

    // Main clock generation
  always begin
    #5 main_clk = ~main_clk;
  end

    // Simulation of a SPI device - Receving
  reg  [3:0] rcv_cnt;
  always @ (posedge spi_sck or spi_ampcs)
  begin
    if(spi_ampcs == 1)
    begin
      rcv_cnt <= 0;
    end
    else begin
      $display($time ,"Bit %d - %b", rcv_cnt, spi_mosi, );
      rcv_cnt = rcv_cnt + 1;
    end
  end

    // Simulation of the ADC
  always @ (posedge spi_sck or spi_adcs)
  begin
    if(spi_adcs == 1)
    begin
      spi_miso <= 1'bz;
    end
    else begin
      spi_miso <= 1'bx;
      #8;
      spi_miso <= ShiftOut[33];
      ShiftOut[33:1] <= ShiftOut[32:0];
      ShiftOut[0] <= 0;
    end
  end
  
    // Test signal state machine
  always @ (posedge main_clk)
  begin
    case (state)
      0:      // Idle
        state <= 0;
      1:      // Simulate a read cycle
      begin
        wSTB_I <= 1;
        wWE_I <= 0;
        state <= 2;
      end
      2:
      begin
        if(wACK_O == 1) 
        begin
          state <= 3;
          wSTB_I <= 0;
        end
      end
      3:
      begin
        state <= 0;
      end
      11:      // Simulate a write cycle
      begin
        wSTB_I <= 1;
        wWE_I <= 1;
        state <= 12;
      end
      12:
      begin
        if(wACK_O == 1) 
        begin
          state <= 13;
          wSTB_I <= 0;
        end
      end
      13:
      begin
        wWE_I <= 0;
        state <= 0;
      end
        
    endcase
  end
    
endmodule

